package de.ozgcloud.nachrichten.common.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.UUID;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.nachrichten.postfach.PostfachTestFactory;
import de.ozgcloud.nachrichten.postfach.osi.MessageTestFactory;

class VorgangServiceTest {

	@InjectMocks
	private VorgangService service;
	@Mock
	private VorgangRemoteService remoteService;

	@DisplayName("Find vorgangIds")
	@Nested
	class TestFindVorgangIds {

		private final Stream<String> returnedVorgangIds = Stream.of("1");
		private final String postfachId = UUID.randomUUID().toString();

		@BeforeEach
		void mock() {
			when(remoteService.findVorgangIds(any(), any())).thenReturn(returnedVorgangIds);
		}

		@Test
		void shouldCallRemoteService() {
			service.findVorgangIds(PostfachTestFactory.POSTFACH_TYPE, postfachId);

			verify(remoteService).findVorgangIds(PostfachTestFactory.POSTFACH_TYPE, postfachId);
		}

		@Test
		void shouldReturnValue() {
			var vorgangIds = service.findVorgangIds(PostfachTestFactory.POSTFACH_TYPE, postfachId);

			assertThat(vorgangIds).isEqualTo(returnedVorgangIds);
		}
	}

	@DisplayName("Get vorgang")
	@Nested
	class TestGetVorgang {

		private final Vorgang returnedVorgang = VorgangTestFactory.create();

		@BeforeEach
		void mock() {
			when(remoteService.getVorgang(any())).thenReturn(returnedVorgang);
		}

		@Test
		void shouldCallRemoteService() {
			service.getVorgang(MessageTestFactory.VORGANG_ID);

			verify(remoteService).getVorgang(MessageTestFactory.VORGANG_ID);
		}

		@Test
		void shouldReturnValue() {
			var vorgang = service.getVorgang(MessageTestFactory.VORGANG_ID);

			assertThat(vorgang).isEqualTo(returnedVorgang);
		}
	}
}
