package de.ozgcloud.nachrichten.postfach.bayernid;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.TestPropertySource;

import de.ozgcloud.common.test.ITCase;

@SpringBootTest(classes = { BayernIdProperties.class })
@ITCase
class BayernIdPropertiesITCase {

	private static final String NAME = "test name";

	@TestPropertySource(properties = {
			"ozgcloud.bayernid.enabled=true",
			"ozgcloud.bayernid.absender.postkorbId=28721c6f-b78f-4d5c-a048-19fd2fc429d2",
			"ozgcloud.bayernid.absender.name=test name",
			"ozgcloud.bayernid.absender.anschrift=Niemalsgasse 5, 99999 Irgendwo Stadt",
			"ozgcloud.bayernid.absender.dienst=Stadtverwaltung",
			"ozgcloud.bayernid.absender.mandant=Fürth",
			"ozgcloud.bayernid.absender.gemeindeSchluessel=09563000",
	})
	@DisplayName("Test loading bayernid configuration")
	@Nested
	class TestLoadingConfiguration {

		// TODO remove access data
		private static final String POSTKORBID = "28721c6f-b78f-4d5c-a048-19fd2fc429d2";
		private static final String DIENST = "Stadtverwaltung";
		private static final String ANSCHRIFT = "Niemalsgasse 5, 99999 Irgendwo Stadt";
		private static final String PASSWORD = "SyPaRAxe97";
		private static final String RESOURCE_PATH = "bayernid.p12";
		private static final String MANDANT = "Fürth";
		private static final String GEMEINDE_SCHLUESSEL = "09563000";

		@Autowired
		private BayernIdProperties properties;

		@Test
		void shouldHaveProperties() {
			assertThat(properties).isNotNull();
		}

		@Test
		void shouldHaveAbsender() {
			assertThat(properties.getAbsender())
					.isNotNull()
					.usingRecursiveComparison()
					.isEqualTo(Absender.builder()
							.anschrift(ANSCHRIFT)
							.dienst(DIENST)
							.name(NAME)
							.mandant(MANDANT)
							.gemeindeSchluessel(GEMEINDE_SCHLUESSEL)
							.postkorbId(POSTKORBID).build());
		}

	}

	@TestPropertySource(properties = { "ozgcloud.other=test name" })
	@DisplayName("Test loading application context without bayerid properties")
	@Nested
	class TestLoadingOtherConfiguration {

		@Value("${ozgcloud.other}")
		private String testValue;

		@Autowired(required = false)
		private BayernIdProperties properties;

		@Test
		void shouldLoadTestProperty() {
			assertThat(testValue).isEqualTo(NAME);
		}

		@Test
		void shouldNotLoadBayernIdProperties() {
			assertThat(properties).isNull();
		}
	}
}
